@@ -1,6 +1,6 @@
 ;;; lang/c99/mach.scm - C parser grammer
 
-;; Copyright (C) 2015-2023 Matthew Wette
+;; Copyright (C) 2015-2025 Matthew Wette
 ;;
 ;; This library is free software; you can redistribute it and/or
 ;; modify it under the terms of the GNU Lesser General Public
@@ -35,7 +35,6 @@
   #:use-module (nyacc lang c99 cpp)
   #:use-module (nyacc lang util)
   #:use-module (nyacc lalr)
-  #:use-module (nyacc parse)
   #:use-module (nyacc lex)
   #:use-module (nyacc util))
 
@@ -50,12 +49,14 @@
 ;; @end deffn
 (define c99-spec
   (lalr-spec
-   (notice (string-append "Copyright (C) 2015-2021 Matthew R. Wette"
+   (notice (string-append "Copyright (C) 2015-2025 Matthew Wette"
 			  license-lgpl3+))
 
    (prec< 'then "else")	       ; "then/else" SR-conflict resolution
    (prec< (left 'imp)	       ; "implied type" SR-conflict resolution
-	  (left "char" "short" "int" "long" "_Fract" "_Accum" "_Sat")
+	  (left "char" "short" "int" "long" "__int128"
+                "signed" "unsigned"
+                "_Fract" "_Accum" "_Sat")
 	  (left "float" "double" "_Complex"))
    (prec< 'shift-on-attr		; living on the edge ...
 	  (nonassoc "__attribute__" "__packed__" "__aligned__" "__alignof__")
@@ -86,10 +87,13 @@
      (postfix-expression "->" identifier ($$ `(i-sel ,$3 ,$1)))
      (postfix-expression "++" ($$ `(post-inc ,$1)))
      (postfix-expression "--" ($$ `(post-dec ,$1)))
-     ("(" type-name ")" "{" initializer-list "}"
-      ($$ `(comp-lit ,$2 ,(tl->list $5))))
-     ("(" type-name ")" "{" initializer-list "," "}"
-      ($$ `(comp-lit ,$2 ,(tl->list $5)))))
+     ("(" type-name ")" "{" literal-list "}" ($$ `(comp-lit ,$2 ,$5)))
+     ("(" attribute-specifier type-name ")" "{" literal-list "}"
+      ($$ `(comp-lit (@ ,(attrl->attrs $2) ,$3 ,$6)))))
+
+    (literal-list
+     (initializer-list)
+     (initializer-list ","))
 
     (argument-expression-list
      (assignment-expression ($$ (make-tl 'expr-list $1)))
@@ -99,8 +103,7 @@
      (arg-expr-hack ($$ (make-tl 'expr-list $1)))
      (argument-expression-list "," arg-expr-hack ($$ (tl-append $1 $3))))
     (arg-expr-hack
-     (declaration-specifiers
-      abstract-declarator ($$ `(param-decl ,$1 ,$2)))
+     (declaration-specifiers abstract-declarator ($$ `(param-decl ,$1 ,$2)))
      (declaration-specifiers ($$ `(param-decl ,$1))))
 
     (unary-expression
@@ -112,7 +115,8 @@
      ("sizeof" "(" type-name ")" ($$ `(sizeof-type ,$3)))
      ("_Alignof" "(" type-name ")" ($$ `(alignof-type ,$3)))
      ("__builtin_offsetof" "(" type-name "," constant-expression ")"
-      ($$ `(offsetof-type ,$3 ,$5))))
+      ($$ `(offsetof-type ,$3 ,$5)))
+     )
 
     (unary-operator ("&" ($$ 'ref-to)) ("*" ($$ 'de-ref))
 		    ("+" ($$ 'pos)) ("-" ($$ 'neg))
@@ -255,6 +259,7 @@
      (fixed-type-specifier ($$ `(type-spec ,$1)))
      (float-type-specifier ($$ `(type-spec ,$1)))
      (fixpt-type-specifier ($$ `(type-spec ,$1)))
+     ("_Sat" fixpt-type-specifier ($$ `(type-spec ,(string-append "_Sat " $2))))
      ("_Bool" ($$/ref 's5.1.5-01 '(type-spec (fixed-type "_Bool"))))
      (complex-type-specifier ($$ `(type-spec ,$1)))
      (struct-or-union-specifier ($$ `(type-spec ,$1)))
@@ -269,45 +274,57 @@
 
     (fixed-type-specifier
      ("short" ($prec 'imp) ($$ '(fixed-type "short")))
-     ("short" "int" ($$ '(fixed-type "short int")))
-     ("signed" "short" ($prec 'imp) ($$ '(fixed-type "signed short")))
-     ("signed" "short" "int" ($$ '(fixed-type "signed short int")))
+     ("short" "int" ($$ '(fixed-type "short")))
+     ("signed" "short" ($prec 'imp) ($$ '(fixed-type "short")))
+     ("signed" "short" "int" ($$ '(fixed-type "short")))
      ("int" ($$ '(fixed-type "int")))
      ("signed" ($prec 'imp) ($$ '(fixed-type "signed")))
-     ("signed" "int" ($$ '(fixed-type "signed int")))
+     ("signed" "int" ($$ '(fixed-type "signed")))
      ("long" ($prec 'imp) ($$ '(fixed-type "long")))
-     ("long" "int" ($$ '(fixed-type "long int")))
-     ("signed" "long" ($prec 'imp) ($$ '(fixed-type "signed long")))
-     ("signed" "long" "int" ($$ '(fixed-type "signed long int")))
+     ("long" "int" ($$ '(fixed-type "long")))
+     ("signed" "long" ($prec 'imp) ($$ '(fixed-type "long")))
+     ("signed" "long" "int" ($$ '(fixed-type "long")))
      ("long" "long" ($prec 'imp) ($$ '(fixed-type "long long")))
-     ("long" "long" "int" ($$ '(fixed-type "long long int")))
-     ("signed" "long" "long" ($prec 'imp)
-      ($$ '(fixed-type "signed long long")))
-     ("signed" "long" "long" "int" ($$ '(fixed-type "signed long long int")))
-     ("unsigned" "short" "int" ($$ '(fixed-type "unsigned short int")))
+     ("long" "long" "int" ($$ '(fixed-type "long long")))
+     ("signed" "long" "long" ($prec 'imp) ($$ '(fixed-type "long long")))
+     ("signed" "long" "long" "int" ($$ '(fixed-type "long long")))
      ("unsigned" "short" ($prec 'imp) ($$ '(fixed-type "unsigned short")))
-     ("unsigned" "int" ($$ '(fixed-type "unsigned int")))
+     ("unsigned" "short" "int" ($$ '(fixed-type "unsigned short")))
      ("unsigned" ($prec 'imp) ($$ '(fixed-type "unsigned")))
-     ("unsigned" "long" "int" ($$ '(fixed-type "unsigned long")))
+     ("unsigned" "int" ($$ '(fixed-type "unsigned")))
      ("unsigned" "long" ($prec 'imp) ($$ '(fixed-type "unsigned long")))
-     ("unsigned" "long" "long" "int"
-      ($$ '(fixed-type "unsigned long long int")))
+     ("unsigned" "long" "int" ($$ '(fixed-type "unsigned long")))
      ("unsigned" "long" "long" ($prec 'imp)
       ($$ '(fixed-type "unsigned long long")))
+     ("unsigned" "long" "long" "int"
+      ($$ '(fixed-type "unsigned long long")))
      ("char" ($$ '(fixed-type "char")))
      ("signed" "char" ($$ '(fixed-type "signed char")))
      ("unsigned" "char" ($$ '(fixed-type "unsigned char")))
-     ("__int128" ($$ '(fixed-type "__int128"))))
+     ("__int128" ($$ '(fixed-type "__int128")))
+     ("unsigned" "__int128" ($$ '(fixed-type "unsigned __int128")))
+     ;; fixes 250627
+     ("short" "signed" ($prec 'imp) ($$ '(fixed-type "short")))
+     ("short" "signed" "int" ($$ '(fixed-type "short")))
+     ("long" "signed" ($prec 'imp) ($$ '(fixed-type "long")))
+     ("long" "signed" "int" ($$ '(fixed-type "long")))
+     ("long" "long" "signed" ($prec 'imp) ($$ '(fixed-type "long long")))
+     ("long" "long" "signed" "int" ($$ '(fixed-type "long long")))
+     ("short" "unsigned" ($prec 'imp) ($$ '(fixed-type "unsigned short")))
+     ("short" "unsigned" "int" ($$ '(fixed-type "unsigned short")))
+     ("long" "unsigned" "int" ($$ '(fixed-type "unsigned long")))
+     ("long" "long" "unsigned" "int" ($$ '(fixed-type "unsigned long long"))))
 
     (float-type-specifier
      ("float" ($prec 'imp) ($$ '(float-type "float")))
      ("double" ($prec 'imp) ($$ '(float-type "double")))
      ("long" "double" ($$ '(float-type "long double")))
      ("_Float16" ($$ '(float-type "_Float16")))
-     ("_Float128" ($$ '(float-type "_Float128"))))
+     ("_Float128" ($$ '(float-type "_Float128")))
+     ("_float16" ($$ '(float-type "_Float16")))
+     ("_float128" ($$ '(float-type "_Float128"))))
 
     (complex-type-specifier
-     ("_Complex" ($$ '(complex-type "_Complex")))
      ("float" "_Complex" ($$ '(complex-type "float _Complex")))
      ("double" "_Complex" ($$ '(complex-type "double _Complex")))
      ("long" "double" "_Complex" ($$ '(complex-type "long double _Complex"))))
@@ -315,49 +332,23 @@
     (fixpt-type-specifier
      ;; http://www.open-std.org/jtc1/sc22/wg21/docs/papers/2001/n1290.pdf
      ("short" "_Fract" ($$ '(fixpt-type "short _Fract")))
+     ("signed" "short" "_Fract" ($$ '(fixpt-type "short _Fract")))
      ("_Fract" ($$ '(fixpt-type "_Fract")))
+     ("signed" "_Fract" ($$ '(fixpt-type "_Fract")))
      ("long" "_Fract" ($$ '(fixpt-type "long _Fract")))
-     ("signed" "short" "_Fract" ($$ '(fixpt-type "signd short _Fract")))
-     ("signed" "_Fract" ($$ '(fixpt-type "signed _Fract")))
-     ("signed" "long" "_Fract" ($$ '(fixpt-type "signed long _Fract")))
+     ("signed" "long" "_Fract" ($$ '(fixpt-type "long _Fract")))
      ("unsigned" "short" "_Fract" ($$ '(fixpt-type "unsigned short _Fract")))
      ("unsigned" "_Fract" ($$ '(fixpt-type "unsigned _Fract")))
      ("unsigned" "long" "_Fract" ($$ '(fixpt-type "unsigned long _Fract")))
      ("short" "_Accum" ($$ '(fixpt-type "short _Accum")))
+     ("signed" "short" "_Accum" ($$ '(fixpt-type "short _Accum")))
      ("_Accum" ($$ '(fixpt-type "_Accum")))
+     ("signed" "_Accum" ($$ '(fixpt-type "_Accum")))
      ("long" "_Accum" ($$ '(fixpt-type "long _Accum")))
-     ("signed" "short" "_Accum" ($$ '(fixpt-type "signd short _Accum")))
-     ("signed" "_Accum" ($$ '(fixpt-type "signed _Accum")))
-     ("signed" "long" "_Accum" ($$ '(fixpt-type "signed long _Accum")))
+     ("signed" "long" "_Accum" ($$ '(fixpt-type "long _Accum")))
      ("unsigned" "short" "_Accum" ($$ '(fixpt-type "unsigned short _Accum")))
      ("unsigned" "_Accum" ($$ '(fixpt-type "unsigned _Accum")))
-     ("unsigned" "long" "_Accum" ($$ '(fixpt-type "unsigned long _Accum")))
-     ("_Sat" "short" "_Fract" ($$ '(fixpt-type "_Sat short _Fract")))
-     ("_Sat" "_Fract" ($$ '(fixpt-type "_Sat _Fract")))
-     ("_Sat" "long" "_Fract" ($$ '(fixpt-type "_Sat long _Fract")))
-     ("_Sat" "signed" "short" "_Fract"
-      ($$ '(fixpt-type "_Sat signd short _Fract")))
-     ("_Sat" "signed" "_Fract" ($$ '(fixpt-type "_Sat signed _Fract")))
-     ("_Sat" "signed" "long" "_Fract"
-      ($$ '(fixpt-type "_Sat signed long _Fract")))
-     ("_Sat" "unsigned" "short" "_Fract"
-      ($$ '(fixpt-type "_Sat unsigned short _Fract")))
-     ("_Sat" "unsigned" "_Fract" ($$ '(fixpt-type "_Sat unsigned _Fract")))
-     ("_Sat" "unsigned" "long" "_Fract"
-      ($$ '(fixpt-type "_Sat unsigned long _Fract")))
-     ("_Sat" "short" "_Accum" ($$ '(fixpt-type "_Sat short _Accum")))
-     ("_Sat" "_Accum" ($$ '(fixpt-type "_Sat _Accum")))
-     ("_Sat" "long" "_Accum" ($$ '(fixpt-type "_Sat long _Accum")))
-     ("_Sat" "signed" "short" "_Accum"
-      ($$ '(fixpt-type "_Sat signd short _Accum")))
-     ("_Sat" "signed" "_Accum" ($$ '(fixpt-type "_Sat signed _Accum")))
-     ("_Sat" "signed" "long" "_Accum"
-      ($$ '(fixpt-type "_Sat signed long _Accum")))
-     ("_Sat" "unsigned" "short" "_Accum"
-      ($$ '(fixpt-type "_Sat unsigned short _Accum")))
-     ("_Sat" "unsigned" "_Accum" ($$ '(fixpt-type "_Sat unsigned _Accum")))
-     ("_Sat" "unsigned" "long" "_Accum"
-      ($$ '(fixpt-type "_Sat unsigned long _Accum"))))
+     ("unsigned" "long" "_Accum" ($$ '(fixpt-type "unsigned long _Accum"))))
 
     ;; This one modified: split out struct-or-union = "struct"|"union"
     (struct-or-union-specifier
@@ -387,9 +378,9 @@
     (struct-declaration-list		; S 6.7.2.1
      (struct-declaration ($$ (make-tl 'field-list $1)))
      (lone-comment ($$ (make-tl 'field-list $1)))
-     ;;(cpp-statement ($$ (make-tl 'field-list $1)))
      (struct-declaration-list struct-declaration ($$ (tl-append $1 $2)))
      (struct-declaration-list lone-comment ($$ (tl-append $1 $2)))
+     ;;(cpp-statement ($$ (make-tl 'field-list $1)))
      ;;(struct-declaration-list cpp-statement ($$ (tl-append $1 $2)))
      ;; Not in C99, but allowed by GNU, I believe:
      (";" ($$ (make-tl 'field-list)))
@@ -461,7 +452,8 @@
     (type-qualifier
      ("const" ($$ `(type-qual (const))))
      ("volatile" ($$ `(type-qual (volatile))))
-     ("restrict" ($$ `(type-qual (restrict)))))
+     ("restrict" ($$ `(type-qual (restrict))))
+     ("_Atomic" ($$ `(type-qual (atomic)))))
 
     (function-specifier
      ("inline" ($$ `(fctn-spec ,$1)))
@@ -555,10 +547,10 @@
 
     (direct-declarator			; S 6.7.6
      (identifier ($$ $1))
-     ;;(ident-like ($$ $1)) ;; generates many SR RR conflicts
      ;;("(" declarator ")" ($$ `(scope ,$2)))
-     ("(" declarator ")" ($$ $2))
+     ("(" declarator ")" ($$ $2)) ;; was `(scope ,$2)
      ("(" attribute-specifier declarator ")" ($$ $3))
+     ;;
      (direct-declarator
       "[" type-qualifier-list assignment-expression "]"
       ($$ `(ary-declr ,$1 ,$3 ,$4)))
@@ -610,7 +602,6 @@
      (declaration-specifiers
       abstract-declarator ($$ `(param-decl ,$1 (param-declr ,$2))))
      (declaration-specifiers
-      ;;($$ `(param-decl ,$1 (param-declr))))
       ($$ `(param-decl ,$1)))
      ;; adding attribute specifiers:
      (declaration-specifiers
@@ -906,7 +897,7 @@
    (hashify-machine
     (make-lalr-machine c99-spec))
    #:keep 2
-   #:keepers '($code-comm $lone-comm $pragma cpp-stmt)))
+   #:keepers '($code-comm $lone-comm $pragma cpp-stmt typename $ident)))
 
 (define c99x-spec (restart-spec c99-spec 'expression))
 
@@ -915,8 +906,7 @@
    (hashify-machine
     (make-lalr-machine c99x-spec))
    #:keep 2
-   ;; Sync w/ #:skip-if-unexp arg to make-lalr-parser in parser.scm.
-   #:keepers '($code-comm $lone-comm $pragma cpp-stmt)))
+   #:keepers '($code-comm $lone-comm $pragma cpp-stmt typename $ident)))
 
 ;;; =====================================
 
