@@ -339,8 +339,6 @@
 ;; broken up in order to not overflow the stack during compilation.
 ;; This uses only syntax-rules; sxml uses syntax-case.
 
-;; update ,_ is "don't care" (so no binding generated)
-
 
 ;; sx-haz-attr? val
 ;;(define (sx-haz-attr? sx)
@@ -413,9 +411,8 @@
 ;; sxm-sexp val pat kt kf
 ;; match sexp @var{val}
 (define-syntax sxm-sexp
-  (syntax-rules (@ unquote _)
+  (syntax-rules (@ unquote)
     ;; accept anything
-    ((_ v (unquote _) kt kf) kt)
     ((_ v (unquote w) kt kf) (let ((w v)) kt))
     ;; capture attributes by name
     ((_ v (tag (@ (ky vl) p1 ...) . nl) kt kf)
@@ -486,9 +483,8 @@
 ;; [ht][vp] = [head,tail][value,pattern]
 ;; Can this be set up to match a string constant?
 (define-syntax sxm-node
-  (syntax-rules (unquote _)
+  (syntax-rules (unquote)
     ((_ v () kt kf) (if (null? v) kt kf))
-    ((_ v (unquote _) kt kf) kt)
     ((_ v (unquote w) kt kf) (let ((w v)) kt))
     ((_ v (hp . tp) kt kf) (if (pair? v) (sxm-sexp v (hp . tp) kt kf) kf))
     ((_ v s kt kf)
